using System.Diagnostics;
using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;
using Microsoft.VisualBasic;
using System.Data;
using System.Collections.Generic;
using System.Threading;
using System.Runtime.InteropServices;



namespace USBTemplate1
{
	sealed class HIDDLLInterface
	{
		// this is the interface to the HID controller DLL - you should not
		// normally need to change anything in this file.
		//
		// WinProc() calls your main form 'event' procedures - these are currently
		// set to..
		//
		// MainForm.OnPlugged(ByVal pHandle as long)
		// MainForm.OnUnplugged(ByVal pHandle as long)
		// MainForm.OnChanged()
		// MainForm.OnRead(ByVal pHandle as long)
		
		
		// HID interface API declarations...
		[DllImport("mcHID.dll",EntryPoint="Connect", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidConnect(int pHostWin);
		[DllImport("mcHID.dll",EntryPoint="Disconnect", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidDisconnect();
		[DllImport("mcHID.dll",EntryPoint="GetItem", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetItem(int pIndex);
		[DllImport("mcHID.dll",EntryPoint="GetItemCount", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetItemCount();
		[DllImport("mcHID.dll",EntryPoint="Read", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidRead(int pHandle, ref byte pData);
		[DllImport("mcHID.dll",EntryPoint="Write", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidWrite(int pHandle, ref byte pData);
		[DllImport("mcHID.dll",EntryPoint="ReadEx", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidReadEx(int pVendorID, int pProductID, ref byte pData);
		[DllImport("mcHID.dll",EntryPoint="WriteEx", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidWriteEx(int pVendorID, int pProductID, ref byte pData);
		[DllImport("mcHID.dll",EntryPoint="GetHandle", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetHandle(int pVendoID, int pProductID);
		[DllImport("mcHID.dll",EntryPoint="GetVendorID", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetVendorID(int pHandle);
		[DllImport("mcHID.dll",EntryPoint="GetProductID", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetProductID(int pHandle);
		[DllImport("mcHID.dll",EntryPoint="GetVersion", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetVersion(int pHandle);
		[DllImport("mcHID.dll",EntryPoint="GetVendorName", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetVendorName(int pHandle, string pText, int pLen);
		[DllImport("mcHID.dll",EntryPoint="GetProductName", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetProductName(int pHandle, string pText, int pLen);
		[DllImport("mcHID.dll",EntryPoint="GetSerialNumber", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetSerialNumber(int pHandle, string pText, int pLen);
		[DllImport("mcHID.dll",EntryPoint="GetInputReportLength", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetInputReportLength(int pHandle);
		[DllImport("mcHID.dll",EntryPoint="GetOutputReportLength", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern int hidGetOutputReportLength(int pHandle);
		[DllImport("mcHID.dll",EntryPoint="SetReadNotify", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern void hidSetReadNotify(int pHandle, bool pValue);
		[DllImport("mcHID.dll",EntryPoint="IsReadNotifyEnabled", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidIsReadNotifyEnabled(int pHandle);
		[DllImport("mcHID.dll",EntryPoint="IsAvailable", ExactSpelling=true, CharSet=CharSet.Ansi, SetLastError=true)]
		public static extern bool hidIsAvailable(int pVendorID, int pProductID);
		
		// windows API declarations - used to set up messaging...

        [DllImport("user32", EntryPoint = "CallWindowProcA", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
        public static extern int CallWindowProc(int lpPrevWndFunc, int hwnd, int Msg, int wParam, int lParam);
        [DllImport("user32", EntryPoint = "SetWindowLongA", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
        public static extern int SetWindowLong(int hwnd, int nIndex, int dwNewLong);

        delegate int SubClassProcDelegate(int hwnd, int msg, int wParam, int lParam);
        [DllImport("USER32.DLL", EntryPoint = "SetWindowLongA", ExactSpelling = true, CharSet = CharSet.Ansi, SetLastError = true)]
        public static extern int DelegateSetWindowLong(int hwnd, int attr, SubClassProcDelegate lval);

		// windows API Constants
		public const int WM_APP = 32768;
		public const short GWL_WNDPROC = - 4;
		
		// HID message constants
		private const decimal WM_HID_EVENT = WM_APP + 200;
		private const short NOTIFY_PLUGGED = 1;
		private const short NOTIFY_UNPLUGGED = 2;
		private const short NOTIFY_CHANGED = 3;
		private const short NOTIFY_READ = 4;
		
		// local variables
		private static int FPrevWinProc; // Handle to previous window procedure
		private static int FWinHandle; // Handle to message window
		private static SubClassProcDelegate Ref_WinProc = new SubClassProcDelegate(WinProc);
		private static object HostForm;
		
		// Set up a windows hook to receive notification
		// messages from the HID controller DLL - then connect
		// to the controller
		public static bool ConnectToHID(Form targetForm)
		{
			int pHostWin = targetForm.Handle.ToInt32();
			FWinHandle = pHostWin;
			pHostWin = hidConnect(FWinHandle);
			FPrevWinProc = DelegateSetWindowLong(FWinHandle, GWL_WNDPROC, Ref_WinProc);
			HostForm = targetForm;
		}
		
		// Unhook from the HID controller and disconnect...
		public static bool DisconnectFromHID()
		{
			bool returnValue;
			returnValue = hidDisconnect();
			SetWindowLong(FWinHandle, GWL_WNDPROC, FPrevWinProc);
			return returnValue;
		}
		
		
		// This is the procedure that intercepts the HID controller messages...
		
		private static int WinProc(int pHWnd, int pMsg, int wParam, int lParam)
		{
			int returnValue;
			if (pMsg == WM_HID_EVENT)
			{
				switch (wParam)
				{
					
					// HID device has been plugged message...
				case NOTIFY_PLUGGED:
					HostForm.OnPlugged(lParam);
					break;
					
					// HID device has been unplugged
				case NOTIFY_UNPLUGGED:
					HostForm.OnUnplugged(lParam);
					break;
					
					// controller has changed...
				case NOTIFY_CHANGED:
					HostForm.OnChanged();
					break;
					
					// read event...
				case NOTIFY_READ:
					HostForm.OnRead(lParam);
					break;
			}
			
		}
		
		// next...
		returnValue = CallWindowProc(FPrevWinProc, pHWnd, pMsg, wParam, lParam);
		
		return returnValue;
	}
}
}
